"""
Hildon UI for Meabook
"""

import gtk
import hildon
import gobject
import gettext
from meabook.ui.ui import MeabookUI

_ = gettext.gettext


def create_button(title, value):
    """Create HildonButton."""

    button = hildon.Button(gtk.HILDON_SIZE_AUTO | \
        gtk.HILDON_SIZE_FINGER_HEIGHT, \
        hildon.BUTTON_ARRANGEMENT_VERTICAL, title, value)
    button.set_style(hildon.BUTTON_STYLE_PICKER)
    button.set_alignment(0, 0, 0, 0)
    return button


class HildonMeabook(MeabookUI):
    def __init__(self, controller):
        MeabookUI.__init__(self, controller)
        self.handler = None
        self.window = hildon.StackableWindow()
        self.window.set_title(_('Meabook'))
        self.window.connect('destroy', self.exit)

        # create menu buttons
        menu = hildon.AppMenu()
        settings_button = hildon.Button(gtk.HILDON_SIZE_AUTO, \
            hildon.BUTTON_ARRANGEMENT_HORIZONTAL, _('Settings'))
        about_button = hildon.Button(gtk.HILDON_SIZE_AUTO, \
            hildon.BUTTON_ARRANGEMENT_HORIZONTAL, _('About'))

        # create filter widgets and connect signals
        self.level1_filter = gtk.RadioButton(None, _('level1'))
        self.level2_filter = gtk.RadioButton(self.level1_filter, _('level2'))
        self.level3_filter = gtk.RadioButton(self.level2_filter, _('level3'))
        for filter_widget in [self.level1_filter, self.level2_filter, \
            self.level3_filter]:
            filter_widget.set_mode(False)
            filter_widget.connect('toggled', self.apply_filter_cb)

        # create items list
        self.selector = hildon.TouchSelector()

        # connect signals
        settings_button.connect('clicked', self.show_settings_dialog_cb)
        about_button.connect('clicked', self.show_about_dialog_cb)
        self.handler = self.selector.connect('changed', self.select_item_cb)

        # packing widgets
        self.window.add(self.selector)
        menu.add_filter(self.level1_filter)
        menu.add_filter(self.level2_filter)
        menu.add_filter(self.level3_filter)
        menu.append(settings_button)
        menu.append(about_button)
        menu.show_all()
        self.window.set_app_menu(menu)
        self.window.show_all()

    def _set_selector_content(self, selector, handler, items):
        """Updates selector content."""

        # temporary block handler
        selector.handler_block(handler)

        # setting new content
        # model: name, internal_name, type
        model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_INT, \
            gobject.TYPE_STRING)
        for name, internal_name, item_type in items:
            _iter = model.append()
            model.set(_iter, 0, name)
            model.set(_iter, 1, internal_name)
            model.set(_iter, 2, item_type)
        selector.remove_column(0)
        selector.append_text_column(model, False)
        selector.unselect_all(0)

        # reconnect callback
        selector.handler_unblock(handler)

    def _show_items_dialog(self, title, items):
        """Creates dialog with items."""

        selector = hildon.TouchSelector()
        window = hildon.StackableWindow()
        window.set_title(title)
        window.add(selector)
        handler = selector.connect('changed', self.select_item_cb)
        self._set_selector_content(selector, handler, items)
        window.show_all()

    # Implementation of Base UI interface
    def start(self):
        self.apply_filter_cb(self.level1_filter)
        gtk.main()

    def exit(self, event):
        gtk.main_quit()
        self.controller.stop()

    def show_about_dialog(self):
        dialog = hildon.Dialog()
        dialog.set_title(_('About'))
        label = gtk.Label('\nMeabook v0.1\n')
        dialog.vbox.add(label)
        dialog.vbox.show_all()
        dialog.run()
        dialog.destroy()

    def show_fields_to_show_dialog(self):
        ConfigurationDialog(self.controller, page=1)

    def show_fileds_order_dialog(self):
        ConfigurationDialog(self.controller, page=2)


    # Hildon UI callbacks
    def show_about_dialog_cb(self, widget):
        """Shows About Dialog."""

        self.show_about_dialog()

    def show_settings_dialog_cb(self, widget):
        """Shows Config dialog."""

        ConfigurationDialog(self.controller)

    def apply_filter_cb(self, widget):
        """Updates toplevel selector with different level items."""

        if not widget.get_active():
            return

        self.window.set_title(' - '.join([_('Meabook'), widget.get_label()]))

        if widget == self.level1_filter:
            self._set_selector_content(self.selector, self.handler, \
                self.controller.get_all_folders())
        elif widget == self.level2_filter:
            self._set_selector_content(self.selector, self.handler, \
                self.controller.get_all_subfolders())
        else:
            self._set_selector_content(self.selector, self.handler, \
                self.controller.get_all_files())

    def select_item_cb(self, widget, column):
        """
        Emits when changes selector content.
        Opens new StackableWindow with new content.
        """

        selected_item_index = widget.get_active(0)
        _iter = widget.get_model(0)[selected_item_index]
        if _iter[2] == 'folder':
            self._show_items_dialog(_iter[0], self.controller.get_items(\
                _iter[1]))
        else:
            self._show_item_dialog(_iter[0], self.controller.get_item(_iter[1]))

    def _show_item_dialog(self, title, item):
        """Shows detailed item information."""

        window = hildon.StackableWindow()
        window.set_title(title)
        pannable_area = hildon.PannableArea()
        vbox = gtk.VBox()

        for key, value in item.items():
            button = create_button(_(key) , value)
            vbox.pack_start(button)

        pannable_area.add_with_viewport(vbox)
        window.add(pannable_area)
        window.show_all()




class ConfigurationDialog:
    """Configuration dialog"""

    def __init__(self, controller, page=1):
        self.controller = controller

        self.window = hildon.StackableWindow()
        self.window.set_title(_('Settings'))
        self.window.connect('destroy', self.close_dialog_cb)

        # create menu
        menu = hildon.AppMenu()
        self.button_order = gtk.RadioButton(None, _('Order'))
        self.button_fields = gtk.RadioButton(self.button_order, _('Fields'))
        self.button_order.set_mode(False)
        self.button_fields.set_mode(False)
        menu.add_filter(self.button_fields)
        menu.add_filter(self.button_order)
        menu.show_all()

        # create other widgets
        self.window.set_app_menu(menu)
        self.selector = hildon.TouchSelector(text=True)
        self.pannable_area = hildon.PannableArea()
        vbox = gtk.VBox()
        vbox.pack_start(self.selector)
        vbox.pack_start(self.pannable_area)
        self.window.add(vbox)

        self.window.show_all()

        # switches to necessary page
        if page == 1:
            self.button_fields.set_active(True)
            self.show_fields_settings_cb(None)
        else:
            self.button_order.set_active(True)
            self.show_order_settings_cb(None)

        # connect signals
        self.button_order.connect('clicked', self.show_order_settings_cb)
        self.button_fields.connect('clicked', self.show_fields_settings_cb)

    def show_fields_settings_cb(self, widget):
        """Shows dialog for selecting fields to show."""

        self.pannable_area.hide()

        fields = self.controller.get_fields()

        # fill items list
        self.selector.get_model(0).clear()
        for field in fields:
            self.selector.append_text(_(field))

        self.selector.set_column_selection_mode( \
                hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
        self.selector.unselect_all(0)

        # mark necessary fields
        fields_to_select = self.controller.config.get_fields()
        model = self.selector.get_model(0)
        for index, field in enumerate(fields):
            if field in fields_to_select:
                self.selector.select_iter(0, model.get_iter(index), False)

        self.selector.show_all()

    def show_order_settings_cb(self, widget):
        """Shows dialog for setting fields order."""


        def show_fields_chooser(widget):
            """Shows dialog to select field from fields list."""

            dialog = hildon.PickerDialog(self.window)
            dialog.set_title(_('Field selection'))
            selector = hildon.TouchSelector(text=True)
            dialog.set_selector(selector)
            selector.set_column_selection_mode( \
                hildon.TOUCH_SELECTOR_SELECTION_MODE_SINGLE)

            # fill fields list
            for field in self.controller.get_fields():
                selector.append_text(field)

            dialog.run()
            widget.set_value(selector.get_current_text())
            dialog.destroy()

        self.selector.hide()

        vbox = gtk.VBox()
        for index, field in enumerate(self.controller.config.get_order()):
            button = create_button(' '.join([_('Position'), str(index)]), \
                _(field))
            button.connect('clicked', show_fields_chooser)
            vbox.pack_start(button)

        self.pannable_area.add_with_viewport(vbox)
        self.pannable_area.show_all()

    def close_dialog_cb(self, event):
        """Save selected options."""

        model = self.selector.get_model(0)
        selected_items = [model[index][0] for index in [item[0] for item in \
            self.selector.get_selected_rows(0)]]
        self.controller.config.set_fields(selected_items)

        try:
            self.controller.config.set_order([button.get_value() for button in \
                self.pannable_area.get_child().get_children()[0].get_children()])
        except AttributeError:
            pass
